@@ -1,14 +1,11 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import shlex
 
 import mock
-from testify import assert_equal
-from testify import setup
-from testify import TestCase
-from testify.assertions import assert_not_equal
 
+from testifycompat import assert_equal
+from testifycompat import assert_not_equal
+from testifycompat import setup
+from testifycompat import TestCase
 from tests.testingutils import autospec_method
 from tron import actioncommand
 from tron.actioncommand import ActionCommand
@@ -16,7 +13,7 @@ from tron.config import schema
 from tron.serialize import filehandler
 
 
-class ActionCommandTestCase(TestCase):
+class TestActionCommand(TestCase):
     @setup
     def setup_command(self):
         self.serializer = mock.create_autospec(filehandler.FileHandleManager)
@@ -111,7 +108,7 @@ class ActionCommandTestCase(TestCase):
         assert self.ac.is_done, self.ac.machine.state
 
 
-class CreateActionCommandFactoryFromConfigTestCase(TestCase):
+class TestCreateActionCommandFactoryFromConfig(TestCase):
     def test_create_default_action_command_no_config(self):
         config = ()
         factory = actioncommand.create_action_runner_factory_from_config(
@@ -120,16 +117,18 @@ class CreateActionCommandFactoryFromConfigTestCase(TestCase):
         assert_equal(type(factory), actioncommand.NoActionRunnerFactory)
 
     def test_create_default_action_command(self):
-        config = schema.ConfigActionRunner('none', None, None)
+        config = schema.ConfigActionRunner(
+            schema.ActionRunnerTypes.none.value, None, None,
+        )
         factory = actioncommand.create_action_runner_factory_from_config(
             config,
         )
-        assert_equal(type(factory), actioncommand.NoActionRunnerFactory)
+        assert type(factory) is actioncommand.NoActionRunnerFactory
 
     def test_create_action_command_with_simple_runner(self):
         status_path, exec_path = '/tmp/what', '/remote/bin'
         config = schema.ConfigActionRunner(
-            'subprocess',
+            schema.ActionRunnerTypes.subprocess.value,
             status_path,
             exec_path,
         )
@@ -140,7 +139,7 @@ class CreateActionCommandFactoryFromConfigTestCase(TestCase):
         assert_equal(factory.exec_path, exec_path)
 
 
-class SubprocessActionRunnerFactoryTestCase(TestCase):
+class TestSubprocessActionRunnerFactory(TestCase):
     @setup
     def setup_factory(self):
         self.status_path = 'status_path'
