@@ -65,6 +65,7 @@ class TestActionRunController:
             self.action_run,
             self.job_run,
         )
+        self.job_run.action_runs.cleanup_action_run = None
 
     def test_handle_command_start_failed(self):
         self.job_run.is_scheduled = True
@@ -105,6 +106,14 @@ class TestActionRunController:
         assert "Attempting to kill" in result
         assert "Warning Message" in result
 
+    def test_handle_retry_default(self):
+        self.controller.handle_command('retry')
+        self.action_run.retry.assert_called_once_with(original_command=True)
+
+    def test_handle_retry_new_command(self):
+        self.controller.handle_command('retry', use_latest_command=True)
+        self.action_run.retry.assert_called_once_with(original_command=False)
+
 
 class TestJobRunController:
     @pytest.fixture(autouse=True)
@@ -189,15 +198,15 @@ class TestConfigController:
         assert resp['hash'] == self.manager.get_hash.return_value
 
     def test_update_config(self):
-        name, content, config_hash = None, mock.Mock(), mock.Mock()
+        name, content, config_hash = 'foo_namespace', mock.Mock(), mock.Mock()
         self.manager.get_hash.return_value = config_hash
         assert not self.controller.update_config(name, content, config_hash)
         self.manager.get_hash.assert_called_with(name)
         self.manager.write_config.assert_called_with(name, content)
-        self.mcp.reconfigure.assert_called_with()
+        self.mcp.reconfigure.assert_called_with(name)
 
     def test_update_config_failure(self):
-        name, content, old_content, config_hash = None, mock.Mock(), mock.Mock(), mock.Mock()
+        name, content, old_content, config_hash = 'foo_namespace', mock.Mock(), mock.Mock(), mock.Mock()
         self.manager.get_hash.return_value = config_hash
         self.manager.write_config.side_effect = [ConfigError("It broke"), None]
         self.controller.read_config = mock.Mock(return_value={'config': old_content})
@@ -209,22 +218,23 @@ class TestConfigController:
         assert error == "It broke"
         self.manager.write_config.call_args_list = [(name, content), (name, old_content)]
         assert self.mcp.reconfigure.call_count == 1
+        self.mcp.reconfigure.assert_called_with(name)
 
     def test_update_config_hash_mismatch(self):
-        name, content, config_hash = None, mock.Mock(), mock.Mock()
+        name, content, config_hash = 'foo_namespace', mock.Mock(), mock.Mock()
         error = self.controller.update_config(name, content, config_hash)
         assert error == "Configuration has changed. Please try again."
 
     def test_delete_config(self):
-        name, content, config_hash = None, "", mock.Mock()
+        name, content, config_hash = 'foo_namespace', "", mock.Mock()
         self.manager.get_hash.return_value = config_hash
         assert not self.controller.delete_config(name, content, config_hash)
         self.manager.delete_config.assert_called_with(name)
-        self.mcp.reconfigure.assert_called_with()
+        self.mcp.reconfigure.assert_called_with(name)
         self.manager.get_hash.assert_called_with(name)
 
     def test_delete_config_failure(self):
-        name, content, config_hash = None, "", mock.Mock()
+        name, content, config_hash = 'foo_namespace', "", mock.Mock()
         self.manager.get_hash.return_value = config_hash
         self.manager.delete_config.side_effect = Exception("some error")
         error = self.controller.delete_config(name, content, config_hash)
@@ -233,12 +243,12 @@ class TestConfigController:
         assert not self.mcp.reconfigure.call_count
 
     def test_delete_config_hash_mismatch(self):
-        name, content, config_hash = None, "", mock.Mock()
+        name, content, config_hash = 'foo_namespace', "", mock.Mock()
         error = self.controller.delete_config(name, content, config_hash)
         assert error == "Configuration has changed. Please try again."
 
     def test_delete_config_content_not_empty(self):
-        name, content, config_hash = None, "content", mock.Mock()
+        name, content, config_hash = 'foo_namespace', "", mock.Mock()
         error = self.controller.delete_config(name, content, config_hash)
         assert error
 
@@ -282,7 +292,7 @@ class TestEventsController:
         assert len(self.eventbus.publish.mock_calls) == 1
 
         self.eventbus.publish.return_value = True
-        assert self.controller.publish(event) == dict(response=f'OK')
+        assert self.controller.publish(event) == dict(response='OK')
         assert len(self.eventbus.publish.mock_calls) == 2
 
     def test_discard(self):
@@ -298,5 +308,5 @@ class TestEventsController:
         assert len(self.eventbus.discard.mock_calls) == 1
 
         self.eventbus.discard.return_value = True
-        assert self.controller.discard(event) == dict(response=f'OK')
+        assert self.controller.discard(event) == dict(response='OK')
         assert len(self.eventbus.discard.mock_calls) == 2
