@@ -320,6 +320,7 @@ class TestJobRun(TestCase):
         self.job_run.output_path = mock.create_autospec(filehandler.OutputPath)
         self.job_run.cleanup()
 
+        self.job_run.notify.assert_called_with(jobrun.JobRun.NOTIFY_REMOVED)
         self.job_run.clear_observers.assert_called_with()
         self.job_run.output_path.delete.assert_called_with()
         assert not self.job_run.node
@@ -338,7 +339,7 @@ class TestJobRun(TestCase):
 class TestJobRunFromState(TestCase):
     @setup
     def setup_jobrun(self):
-        self.action_graph = mock.create_autospec(actiongraph.ActionGraph)
+        self.action_graph = mock.create_autospec(actiongraph.ActionGraph, action_map={})
         self.run_time = datetime.datetime(2012, 3, 14, 15, 9, 26)
         self.path = ['base', 'path']
         self.output_path = mock.create_autospec(filehandler.OutputPath)
@@ -679,6 +680,9 @@ class TestJobRunCollection(TestCase):
         for job_run in job_runs:
             job_run.get_action_run.assert_called_with(action_name)
 
+    def test_get_run_nums(self):
+        assert self.run_collection.get_run_nums() == [5, 4, 3, 2, 1]
+
 
 class TestJobRunStateTransitions:
     """Integration test for the state of a job run when actions change state in various ways."""
@@ -694,9 +698,9 @@ class TestJobRunStateTransitions:
 
     @pytest.fixture
     def job_run(self, tmpdir, mock_event_bus):
-        action_foo = action.Action('foo', 'command', None)
-        action_after_foo = action.Action('after_foo', 'command', None)
-        action_bar = action.Action('bar', 'command', None, triggered_by={'trigger'})
+        action_foo = action.Action('foo', action.ActionCommandConfig('command'), None)
+        action_after_foo = action.Action('after_foo', action.ActionCommandConfig('command'), None)
+        action_bar = action.Action('bar', action.ActionCommandConfig('command'), None, triggered_by={'trigger'})
         action_graph = actiongraph.ActionGraph(
             action_map={
                 'foo': action_foo,
