@@ -5,6 +5,7 @@ import shutil
 import tempfile
 
 import mock
+import pytest
 
 from testifycompat import assert_equal
 from testifycompat import run
@@ -19,11 +20,11 @@ from tron.core.job_collection import JobCollection
 from tron.serialize.runstate import statemanager
 
 
-class TestMasterControlProgram(TestCase):
+class TestMasterControlProgram:
 
     TEST_CONFIG = 'tests/data/test_config.yaml'
 
-    @setup
+    @pytest.fixture(autouse=True)
     def setup_mcp(self):
         self.working_dir = tempfile.mkdtemp()
         self.config_path = tempfile.mkdtemp()
@@ -34,41 +35,57 @@ class TestMasterControlProgram(TestCase):
         self.mcp.state_watcher = mock.create_autospec(
             statemanager.StateChangeWatcher,
         )
-
-    @teardown
-    def teardown_mcp(self):
+        yield
         shutil.rmtree(self.config_path)
         shutil.rmtree(self.working_dir)
 
-    def test_reconfigure(self):
+    def test_reconfigure_default(self):
         autospec_method(self.mcp._load_config)
         self.mcp.state_watcher = mock.MagicMock()
         self.mcp.reconfigure()
-        self.mcp._load_config.assert_called_with(reconfigure=True)
+        self.mcp._load_config.assert_called_with(reconfigure=True, namespace_to_reconfigure=None)
 
-    def test_load_config(self):
+    def test_reconfigure_namespace(self):
+        autospec_method(self.mcp._load_config)
+        self.mcp.state_watcher = mock.MagicMock()
+        self.mcp.reconfigure(namespace='foo')
+        self.mcp._load_config.assert_called_with(reconfigure=True, namespace_to_reconfigure='foo')
+
+    @pytest.mark.parametrize('reconfigure,namespace', [
+        (False, None),
+        (True, None),
+        (True, 'foo'),
+    ])
+    def test_load_config(self, reconfigure, namespace):
         autospec_method(self.mcp.apply_config)
         self.mcp.config = mock.create_autospec(manager.ConfigManager)
-        self.mcp._load_config()
+        self.mcp._load_config(reconfigure, namespace)
         self.mcp.state_watcher.disabled.assert_called_with()
         self.mcp.apply_config.assert_called_with(
             self.mcp.config.load.return_value,
-            reconfigure=False,
+            reconfigure=reconfigure,
+            namespace_to_reconfigure=namespace,
         )
 
+    @pytest.mark.parametrize('reconfigure,namespace', [
+        (False, None),
+        (True, None),
+        (True, 'foo'),
+        (True, 'MASTER'),
+    ])
     @mock.patch('tron.mcp.MesosClusterRepository', autospec=True)
     @mock.patch('tron.mcp.node.NodePoolRepository', autospec=True)
-    def test_apply_config(self, mock_repo, mock_cluster_repo):
+    def test_apply_config(self, mock_repo, mock_cluster_repo, reconfigure, namespace):
         config_container = mock.create_autospec(config_parse.ConfigContainer)
         master_config = config_container.get_master.return_value
-        autospec_method(self.mcp.apply_collection_config)
+        autospec_method(self.mcp.jobs.update_from_config)
         autospec_method(self.mcp.build_job_scheduler_factory)
-        self.mcp.apply_config(config_container)
+        self.mcp.apply_config(config_container, reconfigure, namespace)
         self.mcp.state_watcher.update_from_config.assert_called_with(
             master_config.state_persistence,
         )
         assert_equal(self.mcp.context.base, master_config.command_context)
-        assert_equal(len(self.mcp.apply_collection_config.mock_calls), 1)
+
         mock_repo.update_from_config.assert_called_with(
             master_config.nodes,
             master_config.node_pools,
@@ -79,6 +96,18 @@ class TestMasterControlProgram(TestCase):
         )
         self.mcp.build_job_scheduler_factory(master_config, mock.Mock())
 
+        expected_namespace_to_update = None if namespace == 'MASTER' else namespace
+        self.mcp.jobs.update_from_config.assert_called_once_with(
+            config_container.get_jobs(),
+            self.mcp.build_job_scheduler_factory.return_value,
+            reconfigure,
+            expected_namespace_to_update,
+        )
+        self.mcp.state_watcher.watch_all.assert_called_once_with(
+            self.mcp.jobs.update_from_config.return_value,
+            mock.ANY,
+        )
+
     def test_update_state_watcher_config_changed(self):
         self.mcp.state_watcher.update_from_config.return_value = True
         self.mcp.jobs = mock.create_autospec(JobCollection)
