@@ -6,25 +6,24 @@ import shutil
 import time
 from tempfile import mkdtemp
 from tempfile import NamedTemporaryFile
+from unittest import mock
 
-from testify import assert_equal
-from testify import assert_in
-from testify import assert_not_equal
-from testify import assert_not_in
-from testify import run
-from testify import setup
-from testify import suite
-from testify import teardown
-from testify import TestCase
-from testify import turtle
-
+from testifycompat import assert_equal
+from testifycompat import assert_in
+from testifycompat import assert_not_equal
+from testifycompat import assert_not_in
+from testifycompat import run
+from testifycompat import setup
+from testifycompat import suite
+from testifycompat import teardown
+from testifycompat import TestCase
 from tron.serialize.filehandler import FileHandleManager
 from tron.serialize.filehandler import NullFileHandle
 from tron.serialize.filehandler import OutputPath
 from tron.serialize.filehandler import OutputStreamSerializer
 
 
-class FileHandleWrapperTestCase(TestCase):
+class TestFileHandleWrapper(TestCase):
     @setup
     def setup_fh_wrapper(self):
         self.file = NamedTemporaryFile('r')
@@ -81,12 +80,12 @@ class FileHandleWrapperTestCase(TestCase):
     def test_context_manager(self):
         with self.fh_wrapper as fh:
             fh.write("123")
-        assert fh._fh.closed
+        assert fh._fh is None
         with open(self.file.name) as fh:
             assert_equal(fh.read(), "123")
 
 
-class FileHandleManagerTestCase(TestCase):
+class TestFileHandleManager(TestCase):
     @setup
     def setup_fh_manager(self):
         FileHandleManager.reset()
@@ -219,7 +218,7 @@ class FileHandleManagerTestCase(TestCase):
         )
 
 
-class OutputStreamSerializerTestCase(TestCase):
+class TestOutputStreamSerializer(TestCase):
     @setup
     def setup_serializer(self):
         self.test_dir = mkdtemp()
@@ -227,7 +226,7 @@ class OutputStreamSerializerTestCase(TestCase):
         self.filename = "STARS"
         self.content = "123\n456\n789"
         self.expected = [
-            line.encode('ascii') for line in self.content.split('\n')
+            line for line in self.content.split('\n')
         ]
 
     @teardown
@@ -264,7 +263,7 @@ class OutputStreamSerializerTestCase(TestCase):
         assert_equal(self.serial.tail(file_dne), [])
 
 
-class OutputPathTestCase(TestCase):
+class TestOutputPath(TestCase):
     @setup
     def setup_path(self):
         self.path = OutputPath('one', 'two', 'three')
@@ -309,11 +308,11 @@ class OutputPathTestCase(TestCase):
         assert not os.path.exists(tmp_dir)
 
     def test__eq__(self):
-        other = turtle.Turtle(base='one', parts=['two', 'three'])
+        other = mock.MagicMock(base='one', parts=['two', 'three'])
         assert_equal(self.path, other)
 
     def test__ne__(self):
-        other = turtle.Turtle(base='one/two', parts=['three'])
+        other = mock.MagicMock(base='one/two', parts=['three'])
         assert_not_equal(self.path, other)
 
 
