@@ -100,6 +100,7 @@ class ActionRunAdapter(RunAdapter):
         'action_name',
         'exit_statuses',
         'retries_remaining',
+        'original_command',
     ]
 
     translated_field_names = [
@@ -134,7 +135,7 @@ class ActionRunAdapter(RunAdapter):
         self.include_stderr = include_stderr
 
     def get_raw_command(self):
-        return self._obj.bare_command
+        return self._obj.command_config.command
 
     def get_command(self):
         return self._obj.rendered_command
@@ -248,7 +249,7 @@ class ActionRunGraphAdapter(object):
                 'id': action_run.id,
                 'name': action_run.action_name,
                 'command': action_run.rendered_command,
-                'raw_command': action_run.bare_command,
+                'raw_command': action_run.command_config.command,
                 'state': action_run.state,
                 'start_time': action_run.start_time,
                 'end_time': action_run.end_time,
@@ -408,7 +409,7 @@ class JobIndexAdapter(ReprAdapter):
 
     def get_actions(self):
         def adapt_run(run):
-            return {'name': run.action_name, 'command': run.bare_command}
+            return {'name': run.action_name, 'command': run.command_config.command}
 
         job_run = self._obj.get_runs().get_newest()
         if not job_run:
