@@ -140,7 +140,8 @@ class ActionRunResource(resource.Resource):
 
     @AsyncResource.exclusive
     def render_POST(self, request):
-        return handle_command(request, self.controller, self.action_run)
+        use_latest_command = requestargs.get_bool(request, 'use_latest_command', False)
+        return handle_command(request, self.controller, self.action_run, use_latest_command=use_latest_command)
 
 
 class JobRunResource(resource.Resource):
@@ -491,6 +492,8 @@ class RootResource(resource.Resource):
         self.mcp = mcp
         self.putChild(b'api', ApiRootResource(self.mcp))
         self.putChild(b'web', static.File(web_path))
+        # Temporarily hard-coded while we build the new UI
+        self.putChild(b'web2', static.File(web_path + '2/build'))
         self.putChild(b'', self)
 
     def render_GET(self, request):
