@@ -206,7 +206,8 @@ def get_relevant_run_and_state(job_content):
         job_runs=job_runs,
         job_expected_runtime=job_expected_runtime,
         actions_expected_runtime=actions_expected_runtime,
-        allow_overlap=job_content.get('allow_overlap'),
+        allow_overlap=job_content.get('allow_overlap', False),
+        queueing=job_content.get('queueing', True),
     )
     for run in job_runs:
         state = run.get('state', 'unknown')
@@ -229,17 +230,18 @@ def is_action_failed_or_unknown(job_run):
 
 
 def is_job_stuck(
-    job_runs, job_expected_runtime, actions_expected_runtime, allow_overlap
+    job_runs, job_expected_runtime, actions_expected_runtime, allow_overlap, queueing
 ):
     next_run_time = None
     for job_run in job_runs:
-        if job_run.get('state', 'unknown') == "running":
+        states_to_check = {"running", "waiting"}
+        if job_run.get('state', 'unknown') in states_to_check:
             if is_job_run_exceeding_expected_runtime(
                 job_run, job_expected_runtime
             ):
                 return job_run
             # check if it is still running at next scheduled job run time
-            if not allow_overlap and next_run_time:
+            if not allow_overlap and queueing and next_run_time:
                 difftime = _timestamp_to_timeobj(next_run_time)
                 if time.time() > time.mktime(difftime):
                     return job_run
@@ -254,13 +256,11 @@ def is_job_stuck(
 
 
 def is_job_run_exceeding_expected_runtime(job_run, job_expected_runtime):
+    states_to_check = {"running", "waiting"}
     if job_expected_runtime is not None and job_run.get(
         'state', 'unknown'
-    ) == "running":
+    ) in states_to_check:
         duration_seconds = pytimeparse.parse(job_run.get('duration', ''))
-        # TODO: duration_seconds will be None for a running job if it's root
-        # action is waiting for external dependency. Maybe fix by setting
-        # job's start_time to run_time when that happens.
         if duration_seconds and duration_seconds > job_expected_runtime:
             return True
     return False
@@ -397,6 +397,7 @@ def compute_check_result_for_job(client, job, url_index):
         pmap(job['monitoring']).discard(PRECIOUS_JOB_ATTR)
         .discard('check_every')
         .discard('page_for_expected_runtime')
+        .discard('check_oom_events')
     )
     kwargs = kwargs.update(sensu_kwargs)
     hide_stderr = kwargs.get('hide_stderr', False)
