@@ -106,11 +106,15 @@ class JobContext(object):
         raise KeyError(item)
 
     def _get_date_spec_parts(self, name):
-        parts = name.rsplit(':', 1)
+        parts = name.rsplit('#', 1)
         if len(parts) != 2:
             return name, None
         return parts
 
+    @property
+    def namespace(self):
+        return self.job.name.split(".")[0]
+
 
 class JobRunContext(object):
     def __init__(self, job_run):
@@ -120,6 +124,10 @@ class JobRunContext(object):
     def runid(self):
         return self.job_run.id
 
+    @property
+    def manual(self):
+        return str(self.job_run.manual).lower()
+
     @property
     def cleanup_job_status(self):
         """Provide 'SUCCESS' or 'FAILURE' to a cleanup action context based on
