@@ -2,18 +2,15 @@
 Parse a dictionary structure and return an immutable structure that
 contain a validated configuration.
 """
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import datetime
 import getpass
 import itertools
 import logging
 import os
+from urllib.parse import urlparse
 
 import pytz
-import six
-from six import string_types
+from task_processing.plugins.mesos.constraints import OPERATORS
 
 from tron import command_context
 from tron.config import config_utils
@@ -23,6 +20,7 @@ from tron.config.config_utils import build_dict_name_validator
 from tron.config.config_utils import build_list_of_type_validator
 from tron.config.config_utils import ConfigContext
 from tron.config.config_utils import PartialConfigContext
+from tron.config.config_utils import StringFormatter
 from tron.config.config_utils import valid_bool
 from tron.config.config_utils import valid_dict
 from tron.config.config_utils import valid_float
@@ -35,14 +33,16 @@ from tron.config.schedule_parse import valid_schedule
 from tron.config.schema import CLEANUP_ACTION_NAME
 from tron.config.schema import ConfigAction
 from tron.config.schema import ConfigCleanupAction
+from tron.config.schema import ConfigConstraint
 from tron.config.schema import ConfigJob
+from tron.config.schema import ConfigMesos
+from tron.config.schema import ConfigParameter
 from tron.config.schema import ConfigSSHOptions
 from tron.config.schema import ConfigState
+from tron.config.schema import ConfigVolume
 from tron.config.schema import MASTER_NAMESPACE
 from tron.config.schema import NamedTronConfig
-from tron.config.schema import NotificationOptions
 from tron.config.schema import TronConfig
-from tron.utils.dicts import FrozenDict
 
 log = logging.getLogger(__name__)
 
@@ -62,11 +62,14 @@ def build_format_string_validator(context_object):
         )
 
         try:
-            value % context
+            StringFormatter(context).format(value)
             return value
         except (KeyError, ValueError) as e:
             error_msg = "Unknown context variable %s at %s: %s"
             raise ConfigError(error_msg % (e, config_context.path, value))
+        except (TypeError) as e:
+            error_msg = "Wrong command format %s: %s at %s"
+            raise ConfigError(error_msg % (value, e, config_context.path))
 
     return validator
 
@@ -124,7 +127,7 @@ def valid_known_hosts_file(file_path, config_context):
 
 def valid_command_context(context, config_context):
     # context can be any dict.
-    return FrozenDict(**valid_dict(context or {}, config_context))
+    return valid_dict(context or {}, config_context)
 
 
 def valid_time_zone(tz, config_context):
@@ -145,12 +148,74 @@ def valid_node_name(value, config_context):
     return value
 
 
-def valid_cluster_name(value, config_context):
+def valid_master_address(value, config_context):
+    """Validates and normalizes Mesos master address.
+
+    Must be HTTP or not include a scheme, and only include
+    a host, without any path components.
+    """
     valid_string(value, config_context)
-    if not config_context.partial and value not in config_context.clusters:
-        msg = "Unknown cluster name %s at %s"
-        raise ConfigError(msg % (value, config_context.path))
-    return value
+
+    # Parse with HTTP as default, only HTTP allowed.
+    scheme, netloc, path, params, query, fragment = urlparse(value, 'http')
+    if scheme != 'http':
+        msg = f"Only HTTP supported for Mesos master address, got {value}"
+        raise ConfigError(msg)
+
+    if params or query or fragment:
+        msg = f"Mesos master address may not contain path components, got {value}"
+        raise ConfigError(msg)
+
+    # Only one of netloc or path allowed, and no / except trailing ones.
+    # netloc is empty if there's no scheme, then we try the path.
+    path = path.rstrip('/')
+    if (netloc and path) or '/' in path:
+        msg = f"Mesos master address may not contain path components, got {value}"
+        raise ConfigError(msg)
+
+    if not netloc:
+        netloc = path
+
+    if not netloc:
+        msg = f"Mesos master address is missing host, got {value}"
+        raise ConfigError(msg)
+
+    return f'{scheme}://{netloc}'
+
+
+class ValidateConstraint(Validator):
+    config_class = ConfigConstraint
+    validators = {
+        'attribute': valid_string,
+        'operator': config_utils.build_enum_validator(OPERATORS.keys()),
+        'value': valid_string,
+    }
+
+
+valid_constraint = ValidateConstraint()
+
+
+class ValidateDockerParameter(Validator):
+    config_class = ConfigParameter
+    validators = {
+        'key': valid_string,
+        'value': valid_string,
+    }
+
+
+valid_docker_parameter = ValidateDockerParameter()
+
+
+class ValidateVolume(Validator):
+    config_class = ConfigVolume
+    validators = {
+        'container_path': valid_string,
+        'host_path': valid_string,
+        'mode': config_utils.build_real_enum_validator(schema.VolumeModes),
+    }
+
+
+valid_volume = ValidateVolume()
 
 
 class ValidateSSHOptions(Validator):
@@ -205,15 +270,6 @@ class ValidateSSHOptions(Validator):
 valid_ssh_options = ValidateSSHOptions()
 
 
-class ValidateNotificationOptions(Validator):
-    """Validate notification options."""
-    config_class = NotificationOptions
-    optional = True
-
-
-valid_notification_options = ValidateNotificationOptions()
-
-
 class ValidateNode(Validator):
     config_class = schema.ConfigNode
     validators = {
@@ -230,7 +286,7 @@ class ValidateNode(Validator):
 
     def do_shortcut(self, node):
         """Nodes can be specified with just a hostname string."""
-        if isinstance(node, string_types):
+        if isinstance(node, str):
             return schema.ConfigNode(hostname=node, name=node, **self.defaults)
 
     def set_defaults(self, output_dict, config_context):
@@ -275,6 +331,26 @@ action_context = command_context.build_filled_context(
 )
 
 
+def valid_mesos_action(action, config_context):
+    required_keys = {'cpus', 'mem', 'docker_image'}
+    if action.get('executor') == schema.ExecutorTypes.mesos.value:
+        missing_keys = required_keys - set(action.keys())
+        if missing_keys:
+            raise ConfigError(
+                'Mesos executor for action {id} is missing these required keys: {keys}'.
+                format(
+                    id=action['name'],
+                    keys=missing_keys,
+                ),
+            )
+
+
+def valid_trigger_downstreams(trigger_downstreams, config_context):
+    if isinstance(trigger_downstreams, (type(None), bool, dict)):
+        return trigger_downstreams
+    raise ConfigError('must be None, bool or dict')
+
+
 class ValidateAction(Validator):
     """Validate an action."""
     config_class = ConfigAction
@@ -282,36 +358,75 @@ class ValidateAction(Validator):
     defaults = {
         'node': None,
         'requires': (),
-        'executor': schema.ExecutorTypes.ssh,
-        'cluster': None,
-        'pool': None,
+        'retries': None,
+        'retries_delay': None,
+        'expected_runtime': datetime.timedelta(hours=24),
+        'executor': schema.ExecutorTypes.ssh.value,
         'cpus': None,
         'mem': None,
-        'service': None,
-        'deploy_group': None,
-        'retries': None,
-        'expected_runtime': datetime.timedelta(hours=24),
+        'disk': None,
+        'constraints': None,
+        'docker_image': None,
+        'docker_parameters': None,
+        'env': None,
+        'extra_volumes': None,
+        'trigger_downstreams': None,
+        'triggered_by': None,
+        'on_upstream_rerun': None,
+        'trigger_timeout': None,
     }
     requires = build_list_of_type_validator(
         valid_action_name,
         allow_empty=True,
     )
     validators = {
-        'name': valid_action_name,
-        'command': build_format_string_validator(action_context),
-        'node': valid_node_name,
-        'requires': requires,
-        'executor': config_utils.build_enum_validator(schema.ExecutorTypes),
-        'cluster': valid_cluster_name,
-        'pool': valid_string,
-        'cpus': valid_float,
-        'mem': valid_float,
-        'service': valid_string,
-        'deploy_group': valid_string,
-        'retries': valid_int,
-        'expected_runtime': config_utils.valid_time_delta,
+        'name':
+            valid_action_name,
+        'command':
+            build_format_string_validator(action_context),
+        'node':
+            valid_node_name,
+        'requires':
+            requires,
+        'retries':
+            valid_int,
+        'retries_delay':
+            config_utils.valid_time_delta,
+        'expected_runtime':
+            config_utils.valid_time_delta,
+        'executor':
+            config_utils.build_real_enum_validator(schema.ExecutorTypes),
+        'cpus':
+            valid_float,
+        'mem':
+            valid_float,
+        'disk':
+            valid_float,
+        'constraints':
+            build_list_of_type_validator(valid_constraint, allow_empty=True),
+        'docker_image':
+            valid_string,
+        'docker_parameters':
+            build_list_of_type_validator(
+                valid_docker_parameter, allow_empty=True
+            ),
+        'env':
+            valid_dict,
+        'extra_volumes':
+            build_list_of_type_validator(valid_volume, allow_empty=True),
+        'trigger_downstreams':
+            valid_trigger_downstreams,
+        'triggered_by':
+            build_list_of_type_validator(valid_string, allow_empty=True),
+        'on_upstream_rerun':
+            config_utils.build_real_enum_validator(schema.ActionOnRerun),
+        'trigger_timeout':
+            config_utils.valid_time_delta,
     }
 
+    def post_validation(self, action, config_context):
+        valid_mesos_action(action, config_context)
+
 
 valid_action = ValidateAction()
 
@@ -328,31 +443,69 @@ class ValidateCleanupAction(Validator):
     defaults = {
         'node': None,
         'name': CLEANUP_ACTION_NAME,
-        'executor': schema.ExecutorTypes.ssh,
-        'cluster': None,
-        'pool': None,
+        'retries': None,
+        'retries_delay': None,
+        'expected_runtime': datetime.timedelta(hours=24),
+        'executor': schema.ExecutorTypes.ssh.value,
         'cpus': None,
         'mem': None,
-        'service': None,
-        'deploy_group': None,
-        'retries': None,
-        'expected_runtime': datetime.timedelta(hours=24),
+        'disk': None,
+        'constraints': None,
+        'docker_image': None,
+        'docker_parameters': None,
+        'env': None,
+        'extra_volumes': None,
+        'trigger_downstreams': None,
+        'triggered_by': None,
+        'on_upstream_rerun': None,
+        'trigger_timeout': None,
     }
     validators = {
-        'name': valid_cleanup_action_name,
-        'command': build_format_string_validator(action_context),
-        'node': valid_node_name,
-        'executor': config_utils.build_enum_validator(schema.ExecutorTypes),
-        'cluster': valid_cluster_name,
-        'pool': valid_string,
-        'cpus': valid_float,
-        'mem': valid_float,
-        'service': valid_string,
-        'deploy_group': valid_string,
-        'retries': valid_int,
-        'expected_runtime': config_utils.valid_time_delta,
+        'name':
+            valid_cleanup_action_name,
+        'command':
+            build_format_string_validator(action_context),
+        'node':
+            valid_node_name,
+        'retries':
+            valid_int,
+        'retries_delay':
+            config_utils.valid_time_delta,
+        'expected_runtime':
+            config_utils.valid_time_delta,
+        'executor':
+            config_utils.build_real_enum_validator(schema.ExecutorTypes),
+        'cpus':
+            valid_float,
+        'mem':
+            valid_float,
+        'disk':
+            valid_float,
+        'constraints':
+            build_list_of_type_validator(valid_constraint, allow_empty=True),
+        'docker_image':
+            valid_string,
+        'docker_parameters':
+            build_list_of_type_validator(
+                valid_docker_parameter, allow_empty=True
+            ),
+        'env':
+            valid_dict,
+        'extra_volumes':
+            build_list_of_type_validator(valid_volume, allow_empty=True),
+        'trigger_downstreams':
+            valid_trigger_downstreams,
+        'triggered_by':
+            build_list_of_type_validator(valid_string, allow_empty=True),
+        'on_upstream_rerun':
+            config_utils.build_real_enum_validator(schema.ActionOnRerun),
+        'trigger_timeout':
+            config_utils.valid_time_delta,
     }
 
+    def post_validation(self, action, config_context):
+        valid_mesos_action(action, config_context)
+
 
 valid_cleanup_action = ValidateCleanupAction()
 
@@ -370,8 +523,6 @@ class ValidateJob(Validator):
         'max_runtime': None,
         'monitoring': {},
         'time_zone': None,
-        'service': None,
-        'deploy_group': None,
         'expected_runtime': datetime.timedelta(hours=24),
     }
 
@@ -389,8 +540,6 @@ class ValidateJob(Validator):
         'max_runtime': config_utils.valid_time_delta,
         'monitoring': valid_dict,
         'time_zone': valid_time_zone,
-        'service': valid_string,
-        'deploy_group': valid_string,
         'expected_runtime': config_utils.valid_time_delta,
     }
 
@@ -434,33 +583,8 @@ class ValidateJob(Validator):
 
     def post_validation(self, job, config_context):
         """Validate actions for the job."""
-        incomplete_paasta_actions = []
-
-        def is_incomplete_paasta_action(action):
-            return (
-                action.executor == schema.ExecutorTypes.paasta
-                and (action.service is None or action.deploy_group is None)
-            )
-
-        for _, action in six.iteritems(job['actions']):
+        for _, action in job['actions'].items():
             self._validate_dependencies(job, job['actions'], action)
-            if is_incomplete_paasta_action(action):
-                incomplete_paasta_actions.append(action)
-
-        cleanup_action = job.get('cleanup_action')
-        if cleanup_action and is_incomplete_paasta_action(cleanup_action):
-            incomplete_paasta_actions.append(action)
-
-        if incomplete_paasta_actions and not (
-            job.get('service') and job.get('deploy_group')
-        ):
-            raise ConfigError(
-                'Either job {name} or PaaSTA actions {actions} need a service '
-                'and deploy_group.'.format(
-                    name=job['name'],
-                    actions=incomplete_paasta_actions,
-                ),
-            )
 
 
 valid_job = ValidateJob()
@@ -477,7 +601,7 @@ class ValidateActionRunner(Validator):
 
     validators = {
         'runner_type':
-            config_utils.build_enum_validator(schema.ActionRunnerTypes, ),
+            config_utils.build_real_enum_validator(schema.ActionRunnerTypes),
         'remote_status_path':
             valid_string,
         'remote_exec_path':
@@ -489,16 +613,13 @@ class ValidateStatePersistence(Validator):
     config_class = schema.ConfigState
     defaults = {
         'buffer_size': 1,
-        'connection_details': None,
     }
 
     validators = {
         'name':
             valid_string,
         'store_type':
-            config_utils.build_enum_validator(schema.StatePersistenceTypes, ),
-        'connection_details':
-            valid_string,
+            config_utils.build_real_enum_validator(schema.StatePersistenceTypes),
         'buffer_size':
             valid_int,
     }
@@ -514,6 +635,44 @@ class ValidateStatePersistence(Validator):
 valid_state_persistence = ValidateStatePersistence()
 
 
+class ValidateMesos(Validator):
+    config_class = ConfigMesos
+    option = True
+    defaults = {
+        'master_address': None,
+        'master_port': 5050,
+        'secret_file': None,
+        'role': '*',
+        'principal': 'tron',
+        'enabled': False,
+        'default_volumes': (),
+        'dockercfg_location': None,
+        'offer_timeout': 300,
+    }
+
+    validators = {
+        'master_address':
+            valid_master_address,
+        'master_port':
+            valid_int,
+        'secret':
+            valid_string,
+        'role':
+            valid_string,
+        'enabled':
+            valid_bool,
+        'default_volumes':
+            build_list_of_type_validator(valid_volume, allow_empty=True),
+        'dockercfg_location':
+            valid_string,
+        'offer_timeout':
+            valid_int,
+    }
+
+
+valid_mesos_options = ValidateMesos()
+
+
 def validate_jobs(config, config_context):
     """Validate jobs"""
     valid_jobs = build_dict_name_validator(valid_job, allow_empty=True)
@@ -527,14 +686,18 @@ def validate_jobs(config, config_context):
     config_utils.unique_names(fmt_string, config['jobs'])
 
 
-DEFAULT_STATE_PERSISTENCE = ConfigState('tron_state', 'shelve', None, 1)
+DEFAULT_STATE_PERSISTENCE = ConfigState(
+    name='tron_state',
+    store_type='shelve',
+    buffer_size=1,
+)
 DEFAULT_NODE = ValidateNode().do_shortcut(node='localhost')
 
 
 class ValidateConfig(Validator):
     """Given a parsed config file (should be only basic literals and
     containers), return an immutable, fully populated series of namedtuples and
-    FrozenDicts with all defaults filled in, all valid values, and no unused
+    dicts with all defaults filled in, all valid values, and no unused
     values. Throws a ConfigError if any part of the input dict is invalid.
     """
     config_class = TronConfig
@@ -542,8 +705,7 @@ class ValidateConfig(Validator):
         'action_runner': {},
         'output_stream_dir': None,
         'command_context': {},
-        'ssh_options': ValidateSSHOptions.defaults,
-        'notification_options': None,
+        'ssh_options': ConfigSSHOptions(**ValidateSSHOptions.defaults),
         'time_zone': None,
         'state_persistence': DEFAULT_STATE_PERSISTENCE,
         'nodes': {
@@ -551,22 +713,22 @@ class ValidateConfig(Validator):
         },
         'node_pools': {},
         'jobs': (),
-        'clusters': (),
+        'mesos_options': ConfigMesos(**ValidateMesos.defaults),
+        'eventbus_enabled': None,
     }
     node_pools = build_dict_name_validator(valid_node_pool, allow_empty=True)
     nodes = build_dict_name_validator(valid_node, allow_empty=True)
-    clusters = build_list_of_type_validator(valid_string, allow_empty=True)
     validators = {
         'action_runner': ValidateActionRunner(),
         'output_stream_dir': valid_output_stream_dir,
         'command_context': valid_command_context,
         'ssh_options': valid_ssh_options,
-        'notification_options': valid_notification_options,
         'time_zone': valid_time_zone,
         'state_persistence': valid_state_persistence,
         'nodes': nodes,
         'node_pools': node_pools,
-        'clusters': clusters,
+        'mesos_options': valid_mesos_options,
+        'eventbus_enabled': valid_bool,
     }
     optional = False
 
@@ -575,7 +737,7 @@ class ValidateConfig(Validator):
         another pool.
         """
         all_node_names = set(config['nodes'])
-        for node_pool in six.itervalues(config['node_pools']):
+        for node_pool in config['node_pools'].values():
             invalid_names = set(node_pool.nodes) - all_node_names
             if invalid_names:
                 msg = "NodePool %s contains other NodePools: " % node_pool.name
@@ -595,7 +757,6 @@ class ValidateConfig(Validator):
         config_context = ConfigContext(
             'config',
             node_names,
-            config.get('clusters'),
             config.get('command_context'),
             MASTER_NAMESPACE,
         )
@@ -623,12 +784,18 @@ valid_config = ValidateConfig()
 valid_named_config = ValidateNamedConfig()
 
 
-def validate_fragment(name, fragment):
+def validate_fragment(name, fragment, master_config=None):
     """Validate a fragment with a partial context."""
     config_context = PartialConfigContext(name, name)
     if name == MASTER_NAMESPACE:
         return valid_config(fragment, config_context=config_context)
-    return valid_named_config(fragment, config_context=config_context)
+    if master_config is None:
+        return valid_named_config(fragment, config_context=config_context)
+
+    config_mapping = {MASTER_NAMESPACE: master_config, name: fragment}
+    for config_name, config in validate_config_mapping(config_mapping):
+        if config_name == name:
+            return config
 
 
 def get_nodes_from_master_namespace(master):
@@ -644,11 +811,10 @@ def validate_config_mapping(config_mapping):
     nodes = get_nodes_from_master_namespace(master)
     yield MASTER_NAMESPACE, master
 
-    for name, content in six.iteritems(config_mapping):
+    for name, content in config_mapping.items():
         context = ConfigContext(
             name,
             nodes,
-            master.clusters,
             master.command_context,
             name,
         )
@@ -662,7 +828,7 @@ class ConfigContainer(object):
         self.configs = config_mapping
 
     def items(self):
-        return six.iteritems(self.configs)
+        return self.configs.items()
 
     @classmethod
     def create(cls, config_mapping):
@@ -671,14 +837,14 @@ class ConfigContainer(object):
     # TODO: DRY with get_jobs()
     def get_job_names(self):
         job_names = []
-        for config in six.itervalues(self.configs):
+        for config in self.configs.values():
             job_names.extend(config.jobs)
         return job_names
 
     def get_jobs(self):
         return dict(
             itertools.chain.from_iterable(
-                six.iteritems(config.jobs)
+                config.jobs.items()
                 for _, config in self.configs.items()
             ),
         )
