@@ -1,17 +1,12 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import hashlib
 import logging
 import os
 
-import six
-import yaml as yaml_raw
-
 from tron import yaml
 from tron.config import config_parse
 from tron.config import ConfigError
 from tron.config import schema
+from tron.core.jobgraph import JobGraph
 from tron.utils import maybe_decode
 from tron.utils import maybe_encode
 
@@ -20,8 +15,8 @@ log = logging.getLogger(__name__)
 
 def from_string(content):
     try:
-        return yaml.load(content)
-    except yaml_raw.error.YAMLError as e:
+        return yaml.safe_load(content)
+    except yaml.yaml.error.YAMLError as e:
         raise ConfigError("Invalid config format: %s" % str(e))
 
 
@@ -109,7 +104,12 @@ class ConfigManager(object):
         return read_raw(filename)
 
     def write_config(self, name, content):
-        self.validate_with_fragment(name, from_string(content))
+        self.validate_with_fragment(
+            name,
+            from_string(content),
+            # TODO: remove this constraint after tron triggers across clusters are supported.
+            should_validate_missing_dependency=False,
+        )
         filename = self.get_filename_from_manifest(name)
         write_raw(filename, content)
 
@@ -131,13 +131,24 @@ class ConfigManager(object):
 
         return self.manifest.get_file_name(name) or create_filename()
 
-    def validate_with_fragment(self, name, content):
+    def validate_with_fragment(
+        self,
+        name,
+        content,
+        should_validate_missing_dependency=True,
+    ):
         name_mapping = self.get_config_name_mapping()
         name_mapping[name] = content
-        config_parse.ConfigContainer.create(name_mapping)
+        try:
+            JobGraph(
+                config_parse.ConfigContainer.create(name_mapping),
+                should_validate_missing_dependency=should_validate_missing_dependency,
+            )
+        except ValueError as e:
+            raise ConfigError(str(e))
 
     def get_config_name_mapping(self):
-        seq = six.iteritems(self.manifest.get_file_mapping())
+        seq = self.manifest.get_file_mapping().items()
         return {name: read(filename) for name, filename in seq}
 
     def load(self):
