@@ -6,6 +6,8 @@ import logging
 import os
 from typing import List
 
+from dataclasses import dataclass
+from dataclasses import fields
 from twisted.internet import reactor
 
 from tron import command_context
@@ -91,7 +93,7 @@ class ActionRunFactory(object):
             'job_run_id': job_run.id,
             'name': action.name,
             'node': run_node,
-            'bare_command': action.command,
+            'command_config': action.command_config,
             'parent_context': job_run.context,
             'output_path': job_run.output_path.clone(),
             'cleanup': action.is_cleanup,
@@ -99,14 +101,6 @@ class ActionRunFactory(object):
             'retries_remaining': action.retries,
             'retries_delay': action.retries_delay,
             'executor': action.executor,
-            'cpus': action.cpus,
-            'mem': action.mem,
-            'disk': action.disk,
-            'constraints': action.constraints,
-            'docker_image': action.docker_image,
-            'docker_parameters': action.docker_parameters,
-            'env': action.env,
-            'extra_volumes': action.extra_volumes,
             'trigger_downstreams': action.trigger_downstreams,
             'triggered_by': action.triggered_by,
             'on_upstream_rerun': action.on_upstream_rerun,
@@ -125,6 +119,7 @@ class ActionRunFactory(object):
             'output_path': job_run.output_path.clone(),
             'job_run_node': job_run.node,
             'cleanup': cleanup,
+            'action_graph': job_run.action_graph,
         }
 
         if state_data.get('executor') == ExecutorTypes.mesos.value:
@@ -132,6 +127,41 @@ class ActionRunFactory(object):
         return SSHActionRun.from_state(**args)
 
 
+@dataclass
+class ActionRunAttempt:
+    """Stores state about one try of an action run."""
+    command_config: action.ActionCommandConfig
+    start_time: datetime.datetime = None
+    end_time: datetime.datetime = None
+    rendered_command: str = None
+    exit_status: int = None
+    mesos_task_id: str = None
+
+    def exit(self, exit_status, end_time=None):
+        if self.end_time is None:
+            self.exit_status = exit_status
+            self.end_time = end_time or timeutils.current_time()
+
+    @property
+    def display_command(self):
+        return self.rendered_command or self.command_config.command
+
+    @property
+    def state_data(self):
+        state_data = {
+            'command_config': self.command_config.state_data,
+        }
+        for field in fields(self):
+            if field.name not in state_data:
+                state_data[field.name] = getattr(self, field.name)
+        return state_data
+
+    @classmethod
+    def from_state(cls, state_data):
+        state_data['command_config'] = action.ActionCommandConfig(**state_data['command_config'])
+        return cls(**state_data)
+
+
 class ActionRun(Observable):
     """Base class for tracking the state of a single run of an Action.
 
@@ -254,34 +284,25 @@ class ActionRun(Observable):
         job_run_id,
         name,
         node,
-        bare_command=None,
+        command_config,
         parent_context=None,
         output_path=None,
         cleanup=False,
         start_time=None,
         end_time=None,
         run_state=SCHEDULED,
-        rendered_command=None,
         exit_status=None,
+        attempts=None,
         action_runner=None,
         retries_remaining=None,
         retries_delay=None,
-        exit_statuses=None,
         machine=None,
         executor=None,
-        cpus=None,
-        mem=None,
-        disk=None,
-        constraints=None,
-        docker_image=None,
-        docker_parameters=None,
-        env=None,
-        extra_volumes=None,
-        mesos_task_id=None,
         trigger_downstreams=None,
         triggered_by=None,
         on_upstream_rerun=None,
         trigger_timeout_timestamp=None,
+        original_command=None,
     ):
         super().__init__()
         self.job_run_id = maybe_decode(job_run_id)
@@ -290,38 +311,27 @@ class ActionRun(Observable):
         self.start_time = start_time
         self.end_time = end_time
         self.exit_status = exit_status
-        self.bare_command = maybe_decode(bare_command)
-        self.rendered_command = rendered_command
         self.action_runner = action_runner or NoActionRunnerFactory()
         self.machine = machine or Machine.from_machine(
             ActionRun.STATE_MACHINE, None, run_state
         )
         self.is_cleanup = cleanup
+
         self.executor = executor
-        self.cpus = cpus
-        self.mem = mem
-        self.disk = disk
-        self.constraints = constraints
-        self.docker_image = docker_image
-        self.docker_parameters = docker_parameters
-        self.env = env
-        self.extra_volumes = extra_volumes
-        self.mesos_task_id = mesos_task_id
+        self.command_config = command_config
+        self.original_command = original_command or command_config.command
+        self.attempts = attempts or []
         self.output_path = output_path or filehandler.OutputPath()
         self.output_path.append(self.action_name)
         self.context = command_context.build_context(self, parent_context)
         self.retries_remaining = retries_remaining
         self.retries_delay = retries_delay
-        self.exit_statuses = exit_statuses
         self.trigger_downstreams = trigger_downstreams
         self.triggered_by = triggered_by
         self.on_upstream_rerun = on_upstream_rerun
         self.trigger_timeout_timestamp = trigger_timeout_timestamp
         self.trigger_timeout_call = None
 
-        if self.exit_statuses is None:
-            self.exit_statuses = []
-
         self.action_command = None
         self.in_delay = None
 
@@ -337,6 +347,54 @@ class ActionRun(Observable):
     def name(self):
         return self.action_name
 
+    @property
+    def last_attempt(self):
+        if self.attempts:
+            return self.attempts[-1]
+        return None
+
+    @property
+    def exit_statuses(self):
+        if self.attempts:
+            return [a.exit_status for a in self.attempts if a.end_time]
+        return []
+
+    @property
+    def command(self):
+        if self.attempts:
+            return self.attempts[-1].display_command
+        else:
+            return self.command_config.command
+
+    @property
+    def rendered_command(self):
+        if self.attempts:
+            return self.attempts[-1].rendered_command
+        return None
+
+    @classmethod
+    def attempts_from_state(cls, state_data, command_config_from_state):
+        attempts = []
+        if 'attempts' in state_data:
+            attempts = [ActionRunAttempt.from_state(a) for a in state_data['attempts']]
+        else:
+            rendered_command = maybe_decode(state_data.get('rendered_command'))
+            exit_statuses = state_data.get('exit_statuses', [])
+            # If the action has started, add an attempt for the final try
+            if state_data.get('start_time'):
+                exit_statuses = exit_statuses + [state_data.get('exit_status')]
+            for exit_status in exit_statuses:
+                attempts.append(ActionRunAttempt(
+                    command_config=command_config_from_state,
+                    rendered_command=rendered_command,
+                    exit_status=exit_status,
+                    start_time='unknown',
+                    end_time='unknown',
+                ))
+            if attempts:
+                attempts[-1].mesos_task_id = state_data.get('mesos_task_id')
+        return attempts
+
     @classmethod
     def from_state(
         cls,
@@ -344,6 +402,7 @@ class ActionRun(Observable):
         parent_context,
         output_path,
         job_run_node,
+        action_graph,
         cleanup=False,
     ):
         """Restore the state of this ActionRun from a serialized state."""
@@ -367,33 +426,31 @@ class ActionRun(Observable):
         else:
             action_runner = NoActionRunnerFactory()
 
+        action_config = action_graph.action_map.get(action_name)
+        if action_config:
+            command_config = action_config.command_config
+        else:
+            command_config = action.ActionCommandConfig(command='')
+
+        attempts = cls.attempts_from_state(state_data, command_config)
         run = cls(
             job_run_id=job_run_id,
             name=action_name,
             node=job_run_node,
             parent_context=parent_context,
             output_path=output_path,
-            rendered_command=maybe_decode(state_data.get('rendered_command')),
-            bare_command=maybe_decode(state_data['command']),
+            command_config=command_config,
+            original_command=state_data.get('original_command'),
             cleanup=cleanup,
             start_time=state_data['start_time'],
             end_time=state_data['end_time'],
             run_state=state_data['state'],
             exit_status=state_data.get('exit_status'),
+            attempts=attempts,
             retries_remaining=state_data.get('retries_remaining'),
             retries_delay=state_data.get('retries_delay'),
-            exit_statuses=state_data.get('exit_statuses'),
             action_runner=action_runner,
             executor=state_data.get('executor', ExecutorTypes.ssh.value),
-            cpus=state_data.get('cpus'),
-            mem=state_data.get('mem'),
-            disk=state_data.get('disk'),
-            constraints=state_data.get('constraints'),
-            docker_image=state_data.get('docker_image'),
-            docker_parameters=state_data.get('docker_parameters'),
-            env=state_data.get('env'),
-            extra_volumes=state_data.get('extra_volumes'),
-            mesos_task_id=state_data.get('mesos_task_id'),
             trigger_downstreams=state_data.get('trigger_downstreams'),
             triggered_by=state_data.get('triggered_by'),
             on_upstream_rerun=state_data.get('on_upstream_rerun'),
@@ -406,7 +463,7 @@ class ActionRun(Observable):
             run.transition_and_notify('fail_unknown')
         return run
 
-    def start(self):
+    def start(self, original_command=True):
         """Start this ActionRun."""
         if self.in_delay is not None:
             log.warning(f"{self} cancelling suspend timer")
@@ -416,22 +473,41 @@ class ActionRun(Observable):
         if not self.machine.check('start'):
             return False
 
-        if len(self.exit_statuses) == 0:
+        if len(self.attempts) == 0:
             log.info(f"{self} starting")
         else:
-            log.info(f"{self} restarting, retry {len(self.exit_statuses)}")
+            log.info(f"{self} restarting, retry {len(self.attempts)}")
 
-        self.start_time = timeutils.current_time()
+        new_attempt = self.create_attempt(original_command=original_command)
+        self.start_time = new_attempt.start_time
         self.transition_and_notify('start')
 
-        if not self.is_valid_command:
-            log.error(f"{self} invalid command: {self.bare_command}")
+        if not self.command_config.command:
+            log.error(f"{self} no longer configured in tronfig, cannot run")
+            self.fail(self.EXIT_INVALID_COMMAND)
+
+        if not self.is_valid_command(new_attempt.rendered_command):
+            log.error(f"{self} invalid command: {new_attempt.command_config.command}")
             self.fail(self.EXIT_INVALID_COMMAND)
             return
 
-        return self.submit_command()
+        return self.submit_command(new_attempt)
 
-    def submit_command(self):
+    def create_attempt(self, original_command=True):
+        current_time = timeutils.current_time()
+        command_config = self.command_config.copy()
+        if original_command:
+            command_config.command = self.original_command
+        rendered_command = self.render_command(command_config.command)
+        new_attempt = ActionRunAttempt(
+            command_config=command_config,
+            start_time=current_time,
+            rendered_command=rendered_command,
+        )
+        self.attempts.append(new_attempt)
+        return new_attempt
+
+    def submit_command(self, attempt):
         raise NotImplementedError()
 
     def stop(self):
@@ -450,6 +526,8 @@ class ActionRun(Observable):
             self.clear_trigger_timeout()
             self.exit_status = exit_status
             self.end_time = timeutils.current_time()
+            if self.last_attempt is not None and self.last_attempt.end_time is None:
+                self.last_attempt.exit(exit_status, self.end_time)
             log.info(
                 f"{self} completed with {target}, transitioned to "
                 f"{self.state}, exit status: {exit_status}"
@@ -460,7 +538,7 @@ class ActionRun(Observable):
                 f"{self} cannot transition from {self.state} via {target}"
             )
 
-    def retry(self):
+    def retry(self, original_command=True):
         """Invoked externally (via API) when action needs to be re-tried
         manually.
         """
@@ -476,7 +554,7 @@ class ActionRun(Observable):
 
         if self.is_done:
             self.machine.reset()
-            return self._exit_unsuccessful(self.exit_status)
+            return self._exit_unsuccessful(self.exit_status, retry_original_command=original_command)
         else:
             log.info(f"{self} getting killed for a retry")
             return self.kill(final=False)
@@ -487,8 +565,8 @@ class ActionRun(Observable):
         self.in_delay = None
         self.start()
 
-    def restart(self):
-        """Used by `fail` when action run has to be re-tried."""
+    def restart(self, original_command=True):
+        """Used by `fail` when action run has to be re-tried"""
         if self.retries_delay is not None:
             self.in_delay = reactor.callLater(
                 self.retries_delay.total_seconds(), self.start_after_delay
@@ -497,7 +575,7 @@ class ActionRun(Observable):
             return True
         else:
             self.machine.reset()
-            return self.start()
+            return self.start(original_command=original_command)
 
     def fail(self, exit_status=None):
         if self.retries_remaining:
@@ -505,22 +583,23 @@ class ActionRun(Observable):
 
         return self._done('fail', exit_status)
 
-    def _exit_unsuccessful(self, exit_status=None):
+    def _exit_unsuccessful(self, exit_status=None, retry_original_command=True):
         if self.is_done:
             log.info(
                 f'{self} got exit code {exit_status} but already in terminal '
                 f'state "{self.state}", not retrying',
             )
             return
+        if self.last_attempt is not None:
+            self.last_attempt.exit(exit_status)
         if self.retries_remaining is not None:
             if self.retries_remaining > 0:
                 self.retries_remaining -= 1
-                self.exit_statuses.append(exit_status)
-                return self.restart()
+                return self.restart(original_command=retry_original_command)
             else:
                 log.info(
                     "Reached maximum number of retries: {}".format(
-                        len(self.exit_statuses),
+                        len(self.attempts),
                     )
                 )
         if exit_status is None:
@@ -586,7 +665,6 @@ class ActionRun(Observable):
     @property
     def state_data(self):
         """This data is used to serialize the state of this action run."""
-        rendered_command = self.rendered_command
 
         if isinstance(self.action_runner, NoActionRunnerFactory):
             action_runner = None
@@ -595,64 +673,68 @@ class ActionRun(Observable):
                 status_path=self.action_runner.status_path,
                 exec_path=self.action_runner.exec_path,
             )
-        # Freeze command after it's run
-        command = rendered_command if rendered_command else self.bare_command
+
+        # For temporarily saving old keys.
+        if self.last_attempt:
+            rendered_command = self.last_attempt.rendered_command
+            mesos_task_id = self.last_attempt.mesos_task_id
+        else:
+            rendered_command = None
+            mesos_task_id = None
+        if self.exit_statuses:
+            exit_statuses_without_final = self.exit_statuses[:-1]
+        else:
+            exit_statuses_without_final = []
+
         return {
             'job_run_id': self.job_run_id,
             'action_name': self.action_name,
             'state': self.state,
+            'command_config': self.command_config.state_data,
+            'original_command': self.original_command,
             'start_time': self.start_time,
             'end_time': self.end_time,
-            'command': maybe_decode(command),
-            'rendered_command': maybe_decode(self.rendered_command),
             'node_name': self.node.get_name() if self.node else None,
             'exit_status': self.exit_status,
+            'attempts': [a.state_data for a in self.attempts],
             'retries_remaining': self.retries_remaining,
             'retries_delay': self.retries_delay,
-            'exit_statuses': self.exit_statuses,
             'action_runner': action_runner,
             'executor': self.executor,
-            'cpus': self.cpus,
-            'mem': self.mem,
-            'disk': self.disk,
-            'constraints': self.constraints,
-            'docker_image': self.docker_image,
-            'docker_parameters': self.docker_parameters,
-            'env': self.env,
-            'extra_volumes': self.extra_volumes,
-            'mesos_task_id': self.mesos_task_id,
             'trigger_downstreams': self.trigger_downstreams,
             'triggered_by': self.triggered_by,
             'on_upstream_rerun': self.on_upstream_rerun,
             'trigger_timeout_timestamp': self.trigger_timeout_timestamp,
+            # Temporarily save old keys to make the transition rollback safe.
+            'command': self.command,
+            'rendered_command': rendered_command,
+            'exit_statuses': exit_statuses_without_final,
+            'cpus': self.command_config.cpus,
+            'mem': self.command_config.mem,
+            'disk': self.command_config.disk,
+            'constraints': self.command_config.constraints,
+            'docker_image': self.command_config.docker_image,
+            'docker_parameters': self.command_config.docker_parameters,
+            'env': self.command_config.env,
+            'extra_volumes': self.command_config.extra_volumes,
+            'mesos_task_id': mesos_task_id,
         }
 
     def render_template(self, template):
         """Render our configured command using the command context."""
         return StringFormatter(self.context).format(template)
 
-    def render_command(self):
+    def render_command(self, command):
         """Render our configured command using the command context."""
-        return self.render_template(self.bare_command)
-
-    @property
-    def command(self):
-        if self.rendered_command:
-            return self.rendered_command
         try:
-            self.rendered_command = self.render_command()
+            return self.render_template(command)
         except Exception as e:
             log.error(f"{self} failed rendering command: {e}")
             # Return a command string that will always fail
-            self.rendered_command = self.FAILED_RENDER
-        return self.rendered_command
+            return self.FAILED_RENDER
 
-    @property
-    def is_valid_command(self):
-        """Returns True if the bare_command was rendered without any errors.
-        This has the side effect of actually rendering the bare_command.
-        """
-        return self.command != self.FAILED_RENDER
+    def is_valid_command(self, command):
+        return command != self.FAILED_RENDER
 
     @property
     def is_done(self):
@@ -718,6 +800,14 @@ class ActionRun(Observable):
     def is_blocked_on_trigger(self):
         return not self.is_done and bool(self.remaining_triggers)
 
+    def clear_end_state(self):
+        self.exit_status = None
+        self.end_time = None
+        last_attempt = self.last_attempt
+        if last_attempt:
+            last_attempt.exit_status = None
+            last_attempt.end_time = None
+
     def __getattr__(self, name: str):
         """Support convenience properties for checking if this ActionRun is in
         a specific state (Ex: self.is_running would check if self.state is
@@ -751,8 +841,8 @@ class SSHActionRun(ActionRun, Observer):
         super(SSHActionRun, self).__init__(*args, **kwargs)
         self.recover_tries = 0
 
-    def submit_command(self):
-        action_command = self.build_action_command()
+    def submit_command(self, attempt):
+        action_command = self.build_action_command(attempt)
         try:
             self.node.submit_command(action_command)
         except node.Error as e:
@@ -787,12 +877,12 @@ class SSHActionRun(ActionRun, Observer):
         )
         self.node.submit_command(kill_command)
 
-    def build_action_command(self):
+    def build_action_command(self, attempt):
         """Create a new ActionCommand instance to send to the node."""
         serializer = filehandler.OutputStreamSerializer(self.output_path)
         self.action_command = self.action_runner.create(
             id=self.id,
-            command=self.command,
+            command=attempt.rendered_command,
             serializer=serializer,
         )
         self.watch(self.action_command)
@@ -836,6 +926,12 @@ class SSHActionRun(ActionRun, Observer):
 
     def do_recover(self, delay):
         recovery_command = f"{self.action_runner.exec_path}/recover_batch.py {self.action_runner.status_path}/{self.id}/status"
+        command_config = action.ActionCommandConfig(command=recovery_command)
+        rendered_command = self.render_command(recovery_command)
+        attempt = ActionRunAttempt(
+            command_config=command_config,
+            rendered_command=rendered_command,
+        )
 
         # Put the "recovery" output at the same directory level as the original action_run's output
         self.output_path.parts = []
@@ -846,10 +942,10 @@ class SSHActionRun(ActionRun, Observer):
             job_run_id=self.job_run_id,
             name=f"{self.name}-recovery",
             node=self.node,
-            bare_command=recovery_command,
+            command_config=command_config,
             output_path=self.output_path,
         )
-        recovery_action_command = recovery_run.build_action_command()
+        recovery_action_command = recovery_run.build_action_command(attempt)
         recovery_action_command.write_stdout(
             f"Recovering action run {self.id}",
         )
@@ -862,8 +958,7 @@ class SSHActionRun(ActionRun, Observer):
         # and updates its internal state according to its result.
         self.watch(recovery_action_command)
 
-        self.exit_status = None
-        self.end_time = None
+        self.clear_end_state()
         self.machine.transition('running')
 
         # Still want the action to appear running while we're waiting to submit the recovery
@@ -887,7 +982,7 @@ class SSHActionRun(ActionRun, Observer):
         except node.Error as e:
             log.warning(f"Failed to submit recovery for {self.id}: {e!r}")
 
-    def handle_action_command_state_change(self, action_command, event):
+    def handle_action_command_state_change(self, action_command, event, event_data=None):
         """Observe ActionCommand state changes."""
         log.debug(
             f"{self} action_command state change: {action_command.state}"
@@ -914,32 +1009,33 @@ class SSHActionRun(ActionRun, Observer):
 class MesosActionRun(ActionRun, Observer):
     """An ActionRun that executes the command on a Mesos cluster.
     """
-    def _create_mesos_task(self, mesos_cluster, serializer, task_id=None):
+    def _create_mesos_task(self, mesos_cluster, serializer, attempt, task_id=None):
+        command_config = attempt.command_config
         return mesos_cluster.create_task(
             action_run_id=self.id,
-            command=self.command,
-            cpus=self.cpus,
-            mem=self.mem,
-            disk=1024.0 if self.disk is None else self.disk,
+            command=attempt.rendered_command,
+            cpus=command_config.cpus,
+            mem=command_config.mem,
+            disk=1024.0 if command_config.disk is None else command_config.disk,
             constraints=[[c.attribute, c.operator, c.value]
-                         for c in self.constraints],
-            docker_image=self.docker_image,
-            docker_parameters=[e._asdict() for e in self.docker_parameters],
-            env=build_environment(original_env=self.env, run_id=self.id),
-            extra_volumes=[e._asdict() for e in self.extra_volumes],
+                         for c in command_config.constraints],
+            docker_image=command_config.docker_image,
+            docker_parameters=[e._asdict() for e in command_config.docker_parameters],
+            env=build_environment(original_env=command_config.env, run_id=self.id),
+            extra_volumes=[e._asdict() for e in command_config.extra_volumes],
             serializer=serializer,
             task_id=task_id,
         )
 
-    def submit_command(self):
+    def submit_command(self, attempt):
         serializer = filehandler.OutputStreamSerializer(self.output_path)
         mesos_cluster = MesosClusterRepository.get_cluster()
-        task = self._create_mesos_task(mesos_cluster, serializer)
+        task = self._create_mesos_task(mesos_cluster, serializer, attempt)
         if not task:  # Mesos is disabled
             self.fail(self.EXIT_MESOS_DISABLED)
             return
 
-        self.mesos_task_id = task.get_mesos_id()
+        attempt.mesos_task_id = task.get_mesos_id()
 
         # Watch before submitting, in case submit causes a transition
         self.watch(task)
@@ -954,11 +1050,13 @@ class MesosActionRun(ActionRun, Observer):
             )
             return
 
-        if self.mesos_task_id is None:
+        if not self.attempts or self.attempts[-1].mesos_task_id is None:
             log.error(f'{self} no task ID, cannot recover')
             self.fail_unknown()
             return
 
+        last_attempt = self.attempts[-1]
+
         log.info(f'{self} recovering Mesos run')
 
         serializer = filehandler.OutputStreamSerializer(self.output_path)
@@ -966,12 +1064,13 @@ class MesosActionRun(ActionRun, Observer):
         task = self._create_mesos_task(
             mesos_cluster,
             serializer,
-            self.mesos_task_id,
+            last_attempt,
+            last_attempt.mesos_task_id,
         )
         if not task:
             log.warning(
                 f'{self} cannot recover, Mesos is disabled or '
-                f'invalid task ID {self.mesos_task_id!r}'
+                f'invalid task ID {last_attempt.mesos_task_id!r}'
             )
             self.fail_unknown()
             return
@@ -980,8 +1079,7 @@ class MesosActionRun(ActionRun, Observer):
         mesos_cluster.recover(task)
 
         # Reset status
-        self.exit_status = None
-        self.end_time = None
+        self.clear_end_state()
         self.transition_and_notify('running')
 
         return task
@@ -1012,11 +1110,12 @@ class MesosActionRun(ActionRun, Observer):
             )
 
         mesos_cluster = MesosClusterRepository.get_cluster()
-        if self.mesos_task_id is None:
+        last_attempt = self.last_attempt
+        if last_attempt is None or last_attempt.mesos_task_id is None:
             msgs.append("Error: Can't find task id for the action.")
         else:
-            msgs.append(f"Sending kill for {self.mesos_task_id}...")
-            succeeded = mesos_cluster.kill(self.mesos_task_id)
+            msgs.append(f"Sending kill for {last_attempt.mesos_task_id}...")
+            succeeded = mesos_cluster.kill(last_attempt.mesos_task_id)
             if succeeded:
                 msgs.append(
                     "Sent! It can take up to docker_stop_timeout (current setting is 2 mins) to stop."
@@ -1028,7 +1127,7 @@ class MesosActionRun(ActionRun, Observer):
 
         return '\n'.join(msgs)
 
-    def handle_action_command_state_change(self, action_command, event):
+    def handle_action_command_state_change(self, action_command, event, event_data=None):
         """Observe ActionCommand state changes."""
         log.debug(
             f"{self} action_command state change: {action_command.state}"
@@ -1108,6 +1207,17 @@ class ActionRunCollection(object):
 
     action_runs = property(get_action_runs)
 
+    def update_action_config(self, action_graph):
+        # If there are new command configs that match the action name, update them
+        # Do not update the actual action_graph
+        updated = False
+        for action_run in self.get_action_runs_with_cleanup():
+            new_action = action_graph.action_map.get(action_run.action_name)
+            if new_action and new_action.command_config != action_run.command_config:
+                action_run.command_config = new_action.command_config
+                updated = True
+        return updated
+
     @property
     def cleanup_action_run(self) -> ActionRun:
         return self.run_map.get(action.CLEANUP_ACTION_NAME)
