@@ -1,5 +1,6 @@
 import json
 import logging
+import re
 import socket
 import time
 from urllib.parse import urlparse
@@ -164,10 +165,14 @@ class MesosTask(ActionCommand):
         self.log = self.get_event_logger()
         self.setup_output_logging()
 
+        config_str = str(self.get_config())
+        # AWS_SECRET_ACCESS_KEYs are base64-encoded so it uses alphanumerics plus +, /, and =
+        config_str = re.sub("'AWS_SECRET_ACCESS_KEY': '[a-zA-Z0-9+/=]+'", 'AWS_SECRET_ACCESS_KEY_REDACTED', config_str)
+        config_str = re.sub("'AWS_ACCESS_KEY_ID': '[a-zA-Z0-9]+'", 'AWS_ACCESS_KEY_ID_REDACTED', config_str)
         self.log.info(
             'Mesos task {} created with config {}'.format(
                 self.get_mesos_id(),
-                self.get_config(),
+                config_str,
             ),
         )
 
@@ -202,9 +207,9 @@ class MesosTask(ActionCommand):
 
     def report_resources(self, decrement=False):
         multiplier = -1 if decrement else 1
-        metrics.count(f'tron.mesos.cpus', self.task_config.cpus * multiplier)
-        metrics.count(f'tron.mesos.mem', self.task_config.mem * multiplier)
-        metrics.count(f'tron.mesos.disk', self.task_config.disk * multiplier)
+        metrics.count('tron.mesos.cpus', self.task_config.cpus * multiplier)
+        metrics.count('tron.mesos.mem', self.task_config.mem * multiplier)
+        metrics.count('tron.mesos.disk', self.task_config.disk * multiplier)
 
     def log_event_info(self, event):
         # Separate out so task still transitions even if this nice-to-have logging fails.
@@ -226,13 +231,13 @@ class MesosTask(ActionCommand):
         elif mesos_type is None:
             self.log.info(f'Non-Mesos event: {event.raw}')
             if 'Failed due to offer timeout' in str(event.raw):
-                self.log.info(f'Explanation:')
-                self.log.info(f'This error means that Tron timed out waiting for Mesos to give it the')
-                self.log.info(f'resources requested (ram, cpu, disk, pool, etc).')
-                self.log.info(f'This can happen if the cluster is low on resources, or if the resource')
-                self.log.info(f'requests are too high.')
-                self.log.info(f'Try reducing the resource request, or adding retries + retries_delay.')
-                self.log.info(f'')
+                self.log.info('Explanation:')
+                self.log.info('This error means that Tron timed out waiting for Mesos to give it the')
+                self.log.info('resources requested (ram, cpu, disk, pool, etc).')
+                self.log.info('This can happen if the cluster is low on resources, or if the resource')
+                self.log.info('requests are too high.')
+                self.log.info('Try reducing the resource request, or adding retries + retries_delay.')
+                self.log.info('')
 
         # Mesos events may have task reasons
         if mesos_type:
