@@ -25,9 +25,6 @@ import datetime
 import logging
 import random
 
-from pytz import AmbiguousTimeError
-from pytz import NonExistentTimeError
-
 from tron.config import schedule_parse
 from tron.utils import timeutils
 from tron.utils import trontimespec
@@ -37,16 +34,6 @@ log = logging.getLogger(__name__)
 
 def scheduler_from_config(config, time_zone):
     """A factory for creating a scheduler from a configuration object."""
-    if isinstance(config, schedule_parse.ConfigConstantScheduler):
-        return ConstantScheduler()
-
-    if isinstance(config, schedule_parse.ConfigIntervalScheduler):
-        return IntervalScheduler(
-            interval=config.timedelta,
-            jitter=config.jitter,
-            time_zone=time_zone,
-        )
-
     if isinstance(config, schedule_parse.ConfigGrocScheduler):
         return GeneralScheduler(
             time_zone=time_zone,
@@ -88,32 +75,6 @@ def scheduler_from_config(config, time_zone):
         )
 
 
-class ConstantScheduler(object):
-    """The constant scheduler schedules a new job immediately."""
-    schedule_on_complete = True
-
-    def next_run_time(self, _):
-        return timeutils.current_time()
-
-    def __str__(self):
-        return self.get_name()
-
-    def __eq__(self, other):
-        return isinstance(other, ConstantScheduler)
-
-    def __ne__(self, other):
-        return not self == other
-
-    def get_jitter(self):
-        pass
-
-    def get_name(self):
-        return 'constant'
-
-    def get_value(self):
-        return ''
-
-
 def get_jitter(time_delta):
     if not time_delta:
         return datetime.timedelta()
@@ -186,25 +147,7 @@ class GeneralScheduler(object):
             ) is None:
                 # tz-naive start times need to be localized first to the requested
                 # time zone.
-                try:
-                    start_time = self.time_zone.localize(
-                        start_time,
-                        is_dst=None,
-                    )
-                except AmbiguousTimeError:
-                    # We are in the infamous 1 AM block which happens twice on
-                    # fall-back. Pretend like it's the first time, every time.
-                    start_time = self.time_zone.localize(
-                        start_time,
-                        is_dst=True,
-                    )
-                except NonExistentTimeError:
-                    # We are in the infamous 2:xx AM block which does not
-                    # exist. Pretend like it's the later time, every time.
-                    start_time = self.time_zone.localize(
-                        start_time,
-                        is_dst=True,
-                    )
+                start_time = trontimespec.naive_as_timezone(start_time, self.time_zone)
 
         return self.time_spec.get_match(start_time) + get_jitter(self.jitter)
 
@@ -232,44 +175,3 @@ class GeneralScheduler(object):
 
     def get_value(self):
         return self.original
-
-
-class IntervalScheduler(object):
-    """The interval scheduler runs a job (to success) based on a configured
-    interval.
-    """
-    schedule_on_complete = False
-
-    def __init__(self, interval, jitter, time_zone):
-        self.interval = interval
-        self.jitter = jitter
-        self.time_zone = time_zone
-
-    def next_run_time(self, last_run_time, time_zone=None):
-        last_run_time = last_run_time or timeutils.current_time(tz=time_zone)
-        return last_run_time + self.interval + get_jitter(self.jitter)
-
-    def __str__(self):
-        return "%s %s%s" % (
-            self.get_name(),
-            self.interval,
-            get_jitter_str(self.jitter),
-        )
-
-    def __eq__(self, other):
-        return (
-            isinstance(other, IntervalScheduler)
-            and self.interval == other.interval
-        )
-
-    def __ne__(self, other):
-        return not self == other
-
-    def get_jitter(self):
-        return self.jitter
-
-    def get_name(self):
-        return "interval"
-
-    def get_value(self):
-        return str(self.interval)
