@@ -5,15 +5,14 @@ from __future__ import absolute_import
 from __future__ import unicode_literals
 
 import logging
-import os
 import os.path
 import shutil
+import sys
 import time
 from collections import OrderedDict
 from subprocess import PIPE
 from subprocess import Popen
-
-import six
+from threading import RLock
 
 from tron.utils import maybe_encode
 
@@ -38,12 +37,13 @@ class FileHandleWrapper(object):
     access time and metadata.  These objects should only be created
     by FileHandleManager. Do not instantiate them on their own.
     """
-    __slots__ = ['manager', 'name', 'last_accessed', '_fh']
+    __slots__ = ['manager', 'name', 'last_accessed', '_fh_lock', '_fh']
 
     def __init__(self, manager, name):
         self.manager = manager
         self.name = name
         self.last_accessed = time.time()
+        self._fh_lock = RLock()
         self._fh = NullFileHandle
 
     def close(self):
@@ -52,27 +52,33 @@ class FileHandleWrapper(object):
 
     def close_wrapped(self):
         """Close only the underlying file handle."""
-        self._fh.close()
-        self._fh = NullFileHandle
+        with self._fh_lock:
+            if self._fh is not None:
+                self._fh.close()
+                self._fh = NullFileHandle
 
     def write(self, content):
         """Write content to the fh. Re-open if necessary."""
-        if self._fh == NullFileHandle:
-            try:
-                self._fh = open(self.name, 'ab')
-            except IOError as e:
-                log.error("Failed to open %s: %s", self.name, e)
-                return
+        with self._fh_lock:
+            if self._fh is not None:
+                if self._fh == NullFileHandle:
+                    try:
+                        self._fh = open(self.name, 'ab')
+                    except IOError as e:
+                        log.error("Failed to open %s: %s", self.name, e)
+                        return
 
-        self.last_accessed = time.time()
-        self._fh.write(maybe_encode(content))
-        self.manager.update(self)
+                self.last_accessed = time.time()
+                self._fh.write(maybe_encode(content))
+                self.manager.update(self)
 
     def __enter__(self):
         return self
 
     def __exit__(self, _exc_type, _exc_val, _exc_tb):
-        self.close()
+        with self._fh_lock:
+            self.close()
+            self._fh = None
 
 
 class FileHandleManager(object):
@@ -175,14 +181,14 @@ class OutputStreamSerializer(object):
         if not path or not os.path.exists(path):
             return []
         if not num_lines:
-            num_lines = six.MAXSIZE
+            num_lines = sys.maxsize
 
         try:
             cmd = ('tail', '-n', str(num_lines), path)
             tail_sub = Popen(cmd, stdout=PIPE)
-            return list(line.rstrip() for line in tail_sub.stdout)
+            return list(line.rstrip().decode() for line in tail_sub.stdout)
         except OSError as e:
-            log.error("Could not tail %s: %s" % (path, e))
+            log.error(f"Could not tail {path}: {e}")
             return []
 
     def open(self, filename):
