@@ -5,6 +5,7 @@ from contextlib import contextmanager
 
 from tron.config import schema
 from tron.core import job
+from tron.core import jobrun
 from tron.mesos import MesosClusterRepository
 from tron.serialize import runstate
 from tron.serialize.runstate.dynamodb_state_store import DynamoDBStateStore
@@ -144,6 +145,8 @@ class PersistentStateManager(object):
             self._restore_metadata()
 
         jobs = self._restore_dicts(runstate.JOB_STATE, job_names)
+        for job_name, job_state in jobs.items():
+            job_state['runs'] = self._restore_runs_for_job(job_name, job_state)
         frameworks = self._restore_dicts(runstate.MESOS_STATE, ['frameworks'])
 
         state = {
@@ -152,6 +155,18 @@ class PersistentStateManager(object):
         }
         return state
 
+    def _restore_runs_for_job(self, job_name, job_state):
+        run_nums = job_state['run_nums']
+        runs = []
+        for run_num in run_nums:
+            key = jobrun.get_job_run_id(job_name, run_num)
+            run_state = list(self._restore_dicts(runstate.JOB_RUN_STATE, [key]).values())
+            if not run_state:
+                log.error(f'Failed to restore {key}, no state found for it')
+            else:
+                runs.append(run_state[0])
+        return runs
+
     def _restore_metadata(self):
         metadata = self._impl.restore([self.metadata_key])
         StateMetadata.validate_metadata(metadata.get(self.metadata_key))
@@ -170,6 +185,11 @@ class PersistentStateManager(object):
             for key, state_data in key_to_state_map.items()
         }
 
+    def delete(self, type_enum, name):
+        # A hack to use the save buffer, implementations of save
+        # need to delete if data is None.
+        self.save(type_enum, name, None)
+
     def save(self, type_enum, name, state_data):
         """Persist an items state."""
         key = self._impl.build_key(type_enum, name)
@@ -185,14 +205,11 @@ class PersistentStateManager(object):
         if not key_state_pairs:
             return
 
-        keys = ','.join(str(key) for key, _ in key_state_pairs)
-        log.info("Saving state for %s" % keys)
-
         with self._timeit():
             try:
                 self._impl.save(key_state_pairs)
             except Exception as e:
-                msg = "Failed to save state for %s: %s" % (keys, e)
+                msg = f"Error while saving: {repr(e)}"
                 log.warning(msg)
                 raise PersistenceStoreError(msg)
 
@@ -254,16 +271,34 @@ class StateChangeWatcher(observer.Observer):
         self.config = state_config
         return True
 
-    def handler(self, observable, _event):
+    def handler(self, observable, event, event_data=None):
         """Handle a state change in an observable by saving its state."""
-        if isinstance(observable, job.Job):
-            self.save_job(observable)
-        elif observable == MesosClusterRepository:
+        if observable == MesosClusterRepository:
             self.save_frameworks(observable)
+        elif isinstance(observable, job.Job):
+            if event == job.Job.NOTIFY_NEW_RUN:
+                if event_data is None or not isinstance(event_data, jobrun.JobRun):
+                    log.warning(f'Notified of new run, but no run to watch. Got {event_data}')
+                else:
+                    log.debug(f'Watching new run {event_data}')
+                    self.watch(event_data)
+            else:
+                self.save_job(observable)
+        elif isinstance(observable, jobrun.JobRun):
+            if event == jobrun.JobRun.NOTIFY_REMOVED:
+                self.delete_job_run(observable)
+            else:
+                self.save_job_run(observable)
 
     def save_job(self, job):
         self._save_object(runstate.JOB_STATE, job)
 
+    def save_job_run(self, job_run):
+        self._save_object(runstate.JOB_RUN_STATE, job_run)
+
+    def delete_job_run(self, job_run):
+        self.state_manager.delete(runstate.JOB_RUN_STATE, job_run.name)
+
     def save_frameworks(self, clusters):
         self._save_object(runstate.MESOS_STATE, clusters)
 
