@@ -3,10 +3,9 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+import calendar
 import datetime
 import re
-import time
-from calendar import timegm
 
 
 def current_time(tz=None):
@@ -16,15 +15,7 @@ def current_time(tz=None):
 
 def current_timestamp():
     """Return the current time as a timestamp."""
-    return to_timestamp(current_time())
-
-
-def to_timestamp(time_val):
-    """Generate a unix timestamp for the given datetime instance"""
-    # TODO: replace with datetime.timestamp() after python3.6
-    if time_val.tzinfo:
-        return timegm(time_val.utctimetuple())
-    return time.mktime(time_val.utctimetuple())
+    return current_time().timestamp()
 
 
 def delta_total_seconds(td):
@@ -34,11 +25,11 @@ def delta_total_seconds(td):
     return (microseconds + (seconds + days * 24 * 3600) * 10**6) / 10**6
 
 
-def macro_timedelta(start_date, years=0, months=0, days=0, hours=0):
+def macro_timedelta(start_date, years=0, months=0, days=0, hours=0, minutes=0):
     """Since datetime doesn't provide timedeltas at the year or month level,
     this function generates timedeltas of the appropriate sizes.
     """
-    delta = datetime.timedelta(days=days, hours=hours)
+    delta = datetime.timedelta(days=days, hours=hours, minutes=minutes)
 
     new_month = start_date.month + months
     while new_month > 12:
@@ -47,12 +38,19 @@ def macro_timedelta(start_date, years=0, months=0, days=0, hours=0):
     while new_month < 1:
         new_month += 12
         years -= 1
+    new_year = start_date.year + years
+
+    # TRON-1045: round day to the max days in a given month if the day doesn't
+    # exist for that month. (e.g. Feb 30 rounds to Feb 28 in a non-leap year)
+    _, days_in_month = calendar.monthrange(new_year, new_month)
+    new_day = min(start_date.day, days_in_month)
 
     end_date = datetime.datetime(
-        start_date.year + years,
+        new_year,
         new_month,
-        start_date.day,
+        new_day,
         start_date.hour,
+        start_date.minute,
     )
     month_and_year_delta = end_date - start_date.replace(tzinfo=None)
     delta += month_and_year_delta
@@ -86,6 +84,10 @@ class DateArithmetic(object):
         'day': '%d',
         'hour': '%H',
         'shortdate': '%Y-%m-%d',
+        'ym': '%Y-%m',
+        'ymd': '%Y-%m-%d',
+        'ymdh': '%Y-%m-%dT%H',
+        'ymdhm': '%Y-%m-%dT%H:%M',
     }
 
     @classmethod
@@ -109,8 +111,14 @@ class DateArithmetic(object):
                 dt += macro_timedelta(dt, **kwargs)
             return dt.strftime(cls.DATE_FORMATS[attr])
 
+        if attr in ('ym', 'ymd', 'ymdh', 'ymdhm'):
+            args = [0] * len(attr)
+            args[-1] = delta
+            dt += macro_timedelta(dt, *args)
+            return dt.strftime(cls.DATE_FORMATS[attr])
+
         if attr == 'unixtime':
-            return int(to_timestamp(dt)) + delta
+            return int(dt.timestamp()) + delta
 
         if attr == 'daynumber':
             return dt.toordinal() + delta
